
/*
   Control of step motor, which moves the Digicomp clock plate.
   Orientation:
   Digicomp seen from front, onto logic rods.
   "Left" = coordinate 0 = clock plate moved "IN" toward digit display
   "Right" = clock plate moved "OUT" away from digit display

   Arduino UNO R3 pins:
  pin 2 - out - grün - Stepper enable
  pin 3 - out - weiss - Stepper step
  pin 4 - out - violett - Stepper direction
  pin 5 - in -  ??? - Endschalter links (bei Ziffernanzeige)
  pin 6 - in -  ??? - Endschalter rechts (bei Clock-Griff)


*/
#ifndef POSITIONER_H
#define POSITIONER_H

#include "operatorswitches.h"

extern OperatorSwitches operatorSwitches ;

class Positioner {
  private:
    // raw GPIOs
    static const int pinEnable = 2 ;
    static const int pinStep = 3 ;
    static const int pinDirection = 4 ;
    static const int pinEndSwitchLeft = 5 ;
    static const int pinEndSwitchRight = 6 ;

    // micro stepping enabled? 1 pulse = 1/n steps
    // Jumpers on A4988 module (IN=HIGH)
    // MS1  MS2  MS3  steps
    // out  out  out  1
    // IN   out  out  1/2
    // out  IN   out  1/4
    // IN   IN   out  1/8
    // IN   IN   IN   1/16

    // max step resolution results in minimal noise
    static const int pulsesPerSteps = 16 ;


  public:
    // predefined move directions
    enum Direction { directionNone, directionRight, directionLeft} ;

    // predefined move speeds (in microscond waittime between steps
    enum Speed {
      speedSlow = 20000, // 20000
      speedFast = 7500
                  //      speedSlow = 2500,
                  //speedFast = 500
    } ;

    int position ; // global position in steps, 0=left end switch

    // reached left end switch?
    bool atLeft()  {
      // switch closes to GND
      return digitalRead(pinEndSwitchLeft) == 0 ;
    }

    // reached right end switch?
    bool atRight() {
      // switch closes to GND
      return digitalRead(pinEndSwitchRight) == 0 ;
    }

    /* Stepper idle or holding position
    */
    void engageStepper(bool active) {
      digitalWrite(pinEnable, active ? LOW : HIGH);
    }

    /* move()
       travel, until end stop hit or another operator switch changed
       returns stop reason
    */

    // why did a move stopped?
    enum StopReason {
      none,
      endSwitch, // hit an end switch
      operatorSwitch // operator switch touched by user
    } ;

    enum StopReason move(enum Direction direction, int distance, enum Speed stepTimeUs)
    {
      if (direction == directionNone)
        return none ;
      // calculate timing phases
      bool endless  ; // distance invalid, no limit
      int timeStrobeUs = 10 ; // step impulse
      int timeIdleUs = stepTimeUs - timeStrobeUs ;
      if (timeIdleUs < 0)
        timeIdleUs = 0 ;
      // faster micro steps
      timeIdleUs /= pulsesPerSteps ;

      endless = (distance <= 0) ;
      // more pulses needed if micro stepping
      distance *= pulsesPerSteps ;

      digitalWrite(pinEnable, LOW); // activate drivers
      if (direction == directionLeft)
        digitalWrite(pinDirection, 0);
      else
        digitalWrite(pinDirection, 1);
      // separate code paths for move directions, speed

      while (distance > 0 || endless) {
        // abort on switch change
        if (operatorSwitches.anyChange())
          return operatorSwitch ;
        if (direction == directionLeft) {
          if (atLeft()) {
            position = 0 ;
            return endSwitch ;
          }
          position-- ;
        } else {
          // directionRight
          if (atRight())
            return endSwitch ;
          position++ ;
        }
        // one stepper pulse:
        // 1) strobe STEP
        digitalWrite(pinStep, HIGH);
        delayMicroseconds(timeStrobeUs) ;
        digitalWrite(pinStep, LOW);
        // 2) wait
        delayMicroseconds(timeIdleUs) ;
        distance-- ;
      }
      return none ; // no prematurely stop
    }

    /* calibrate()
       move clock plate to left position
    */
    void calibrate() {
      // calibrate: find left switch
      if (atLeft())
        // move away from left switch first
        move(directionRight, 100, speedSlow) ;
      while (!atLeft()) {
        move(directionLeft, 5, speedSlow) ;
      }
      position = 0 ; // now leftmost
    }

    /* setup()
    */
    void setup(void) {
      pinMode(pinDirection, OUTPUT);
      pinMode(pinStep, OUTPUT);
      pinMode(pinEnable, OUTPUT);
      pinMode(pinEndSwitchLeft, INPUT_PULLUP);
      pinMode(pinEndSwitchRight, INPUT_PULLUP);

      // disable motor driver
      digitalWrite(pinEnable, LOW);

    }
} ;


#endif // POSITIONER_H
